/* *****************************************************************************
 *
 * Filename: MonthImage.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  17 December 2002, 11:00
 *
 * Purpose:  Bean for displaying an image for each month of the year.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.beans.Beans;
import java.awt.*;
import java.awt.image.*;
import java.util.ResourceBundle;
import javax.swing.JComponent;
import javax.accessibility.*;

import com.tsac.calmaker.IDateable;
import com.tsac.calmaker.IScaleable;
import com.tsac.calmaker.MainFrame;

public class MonthImage
    extends javax.swing.JComponent
    implements IDateable, IScaleable, Accessible
{
    // Statics
    protected static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    public static final String IMAGES_PROPERTY = "images";
    
    // Properties
    protected String[] _filenames = new String[12];
    
    // State
    protected transient int _currmonth;
    protected transient MediaTracker tracker = new MediaTracker( this );
    protected transient Image[] _images = new Image[12];
    protected transient ImageObserver _observer;
    
    // Cached
    private transient Font _font = new Font( "Helvetica", Font.PLAIN, 12 );
    private transient FontMetrics _fontMetrics;
    

    // Contructor
	public MonthImage()
	{
        setFocusable( true );
        setPreferredSize( new Dimension( 200, 200 ) );
	    _fontMetrics = getFontMetrics( _font );
	    
        if( !Beans.isDesignTime() )
        {
            // For testing only!
            setImages( new String[] {
                "images/january.jpg", "images/february.jpg", "images/march.jpg",
                "images/april.jpg",   "images/may.jpg",      "images/june.jpg",
                "images/july.jpg",    "images/august.jpg",   "images/september.jpg",
                "images/october.jpg", "images/november.jpg", "images/december.jpg"
            } );
	        
            // Need an observer to refresh the display after the image loads
	        _observer = new ImageObserver() {
	            public boolean imageUpdate( Image img, int infoflags, int x, int y, int width, int height )
	            {
	                if( (infoflags & ImageObserver.ALLBITS) != 0 )
	                {
	                    repaint();
	                    return false;
	                }
	                
	                return true;
	            }
	        };
	    }
	}


    // Property accessors
    public String[] getImages()  { return _filenames; }
    public String getImages( int n )  { return _filenames[n]; }
    
    public void setImages( String[] i )
    {
        // Suppress if no change
        if( _filenames == i )
            return;

        String[] old = _filenames;
        
        // Remove old images
        for( int n = 0; n < _filenames.length; n++ )
            removeImage( n );

        // Store new image array
        _filenames = i;

        // Load new images
        for( int n = 0; n < _filenames.length; n++ )
            loadImage( n );
        
        // Fire a property change with old and new values
        firePropertyChange( IMAGES_PROPERTY, old, _filenames );
    }
    
    public void setImages( int n, String i )
    {
        // Suppress if no change (object comparison)
        if( _filenames[n] == i )
            return;
        
        // If this is not the same image as before (but with a different string object)?
        if( (i == null) || (_filenames[n] == null) || !_filenames[n].equals(i) )
        {
            removeImage( n );
            _filenames[n] = i;
            loadImage( n );
            repaint();
        }
                
        // Store the new filename anyway (could be a different string)
        _filenames[n] = i;
        
        // Fire a property change (don't bother sending the old state)
        firePropertyChange( IMAGES_PROPERTY, null, _filenames );
    }


    // From IDateable
    public void setDate( int year, int month )
    {
        _currmonth = month;
        repaint();
    }
    
    
    public void paint( Graphics g )
    {
        paint( g, MainFrame.scale );
    }


    // Overrides default paint method
    public void paint( Graphics g, double scale )
    {
        if( Beans.isDesignTime() )
        {
            g.setColor( Color.lightGray );
            g.fillRect( 0, 0, getWidth(), getHeight() );
            g.setColor( Color.black );
            g.drawRect( 2, 2, getWidth() - 4, getHeight() - 4 );
            g.drawString( getClass().getName(), 4, _fontMetrics.getAscent() + 4 );
            return;
        }
        
        if( (_images != null) && (_images[_currmonth] != null) )
        {
            int w = _images[_currmonth].getWidth( _observer );
            int h = _images[_currmonth].getHeight( _observer );
            
            int status = tracker.statusID( _currmonth, true );
            
            int width  = (int) (scale * getWidth());
            int height = (int) (scale * getHeight());
            
            if( status == MediaTracker.COMPLETE )
            {
                double xscale = (double) getWidth() / (double) w;
                double yscale = (double) getHeight() / (double) h;
                        
                double actscale = Math.min( xscale, yscale );
                
                int imgwidth = (int) ((double) w * actscale * scale);
                int imgheight = (int) ((double) h * actscale * scale);
                                
                g.drawImage( _images[_currmonth],
                    (width - imgwidth)/2, (height - imgheight)/2,
                    imgwidth, imgheight, Color.white, _observer );
            }
            else if( status == MediaTracker.LOADING )
            {
                g.setColor( Color.black );
                g.setFont( _font );
                g.drawString( "Loading...",
                    (width - _fontMetrics.stringWidth("Loading..."))/2,
                    (height - _fontMetrics.getHeight())/2 );
            }
            else
            {
                g.setColor( Color.black );
                g.setFont( _font );
                g.drawString( "Failed!",
                    (width - _fontMetrics.stringWidth("Failed!"))/2,
                    (height - _fontMetrics.getHeight())/2 );
            }
        }
    }
    
    
    private void loadImage( int n )
    {
        // Is the image we are supposed to load unset?
        if( _filenames[n] == null )
        {
            _images[n] = null;
            return;
        }

        // Load the image and store it in the images array
        _images[n] = Toolkit.getDefaultToolkit().getImage( _filenames[n] );
        tracker.addImage( _images[n], n );
    }
    
    
    private void removeImage( int n )
    {
        // Do we have an existing image to remove?
        if( _images[n] != null )
            tracker.removeImage( _images[n] );
        
        // Remember the fact, so that we won't remove it again
        _images[n] = null;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // Accessibility stuff
    
    public class AccessibleMonthImage
        extends JComponent.AccessibleJComponent
    {
        public String getAccessibleName()
        {
            // Use name set by the developer if set, otherwise use name from bean
            String name = super.getAccessibleName();            
            return (name != null) ? name : resources.getString( "MonthImage_Name" );
        }
        
        public String getAccessibleDescription()
        {
            // Use description set by the developer if set, otherwise use description from bean
            String name = super.getAccessibleDescription();            
            return (name != null) ? name : resources.getString( "MonthImage_Desc" );
        }
    }
    
    
    public AccessibleContext getAccessibleContext()
    {
        if( accessibleContext == null )
            accessibleContext = new AccessibleMonthImage();
        
        return accessibleContext;
    }
}