/* *****************************************************************************
 *
 * Filename: Utilities.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  01 February 2003
 *
 * Purpose:  Collection of useful miscellaneous methods.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.Component;
import java.awt.Container;
import java.awt.GraphicsEnvironment;
import java.util.Arrays;
import java.util.List;
import javax.swing.AbstractButton;
import javax.swing.JLabel;
import javax.accessibility.AccessibleContext;

/**
 *
 * @author  Robert
 */
public class Utilities
{
    // Constants
    public static final int SERIF      = 0;
    public static final int SANS_SERIF = 1;
    public static final int MONOSPACED = 2;
    
    
    /**
     * Simple recursive method for cleaning up label names for accessibility.
     * Just removes the trailing colon and sets it as the accessible name.  Is
     * optimised slightly by not treating JLabels as containers.
     */
    public static void setAccessibleLabelNames( Container parent )
    {
        Component[] children = parent.getComponents();
        
        for( int n = children.length-1; n >= 0; n-- )
        {
            if( children[n] instanceof JLabel )
            {
                JLabel label = (JLabel) children[n];
                String text = label.getText();
                if( text.charAt(text.length()-1) == ':' )
                    text = text.substring( 0, text.length()-1 );
                label.getAccessibleContext().setAccessibleName( text );
            }
            else if( children[n] instanceof Container )
                setAccessibleLabelNames( (Container) children[n] );
        }
    }
    
    
    /**
     * Simple recursive method for cleaning up button names for accessibility.
     * Just removes the trailing ellipsis and re-sets it as the accessible name.
     * Is optimised slightly by not treating buttons as containers.
     */
    public static void fixAccessibleButtonNames( Container parent )
    {
        Component[] children = parent.getComponents();
        
        for( int n = children.length-1; n >= 0; n-- )
        {
            if( children[n] instanceof AbstractButton )
            {
                AbstractButton button = (AbstractButton) children[n];
                AccessibleContext ctx = button.getAccessibleContext();
                String text = ctx.getAccessibleName();
                if( text.endsWith( "..." ) )
                    text = text.substring( 0, text.length()-3 );
                ctx.setAccessibleName( text );
            }
            else if( children[n] instanceof Container )
                fixAccessibleButtonNames( (Container) children[n] );
        }
    }
    
    
    // Table of fonts and their possible equivalents
    private static final String[][] equivalents = new String[][] {
        { "Serif", "Times" },
        { "SansSerif", "Helvetica", "Arial" },
        { "Monospaced", "Courier New" }
    };
    
    /**
     * Finds a suitable font to use for a given type of font e.g. sans-serif.
     */
    public static String findFont( int type )
    {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        List fonts = Arrays.asList( ge.getAvailableFontFamilyNames() );
        
        for( int n = 0; n < equivalents[type].length; n++ )
        {
            if( fonts.contains( equivalents[type][n] ) )
                return equivalents[type][n];
        }
        
        return equivalents[type][0];
    }
}
